# Generated by roxygen2: do not edit by hand

S3method("+",bbplot)
S3method("<=",bb_theme)
S3method(bbplot_add,"function")
S3method(bbplot_add,bb_facet_wrap)
S3method(bbplot_add,bb_grid)
S3method(bbplot_add,bb_labs)
S3method(bbplot_add,bb_layer)
S3method(bbplot_add,bb_legend)
S3method(bbplot_add,bb_palette)
S3method(bbplot_add,bb_scale)
S3method(bbplot_add,bb_theme)
S3method(bbplot_add,expression)
S3method(bbplot_add,formula)
S3method(print,bbplot)
S3method(print,bbplot_layer_list)
export("%>%")
export(as.bbplot)
export(bb_aes)
export(bb_bar)
export(bb_boxplot)
export(bb_density)
export(bb_errorbar)
export(bb_facet_wrap)
export(bb_grid)
export(bb_hist)
export(bb_labs)
export(bb_legend)
export(bb_line)
export(bb_lm)
export(bb_point)
export(bb_scale_cex_continuous)
export(bb_scale_cex_manual)
export(bb_scale_col_gradient)
export(bb_scale_col_manual)
export(bb_scale_col_palette)
export(bb_scale_lty_manual)
export(bb_scale_pch_manual)
export(bb_segment)
export(bb_signif)
export(bb_sub)
export(bb_text)
export(bb_theme)
export(bb_theme_bw)
export(bb_theme_deepblue)
export(bb_theme_expand)
export(bb_theme_grey)
export(bb_theme_minimal)
export(bb_tile)
export(bb_title)
export(bb_xlab)
export(bb_ylab)
export(bbplot)
export(bbplot_add)
export(set_bb_theme)
export(unset_bb_theme)
importFrom(graphics,boxplot)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(rlang,enquos)
importFrom(rlang,quo_name)
importFrom(scales,col_factor)
importFrom(scales,col_numeric)
importFrom(stats,density)
importFrom(utils,modifyList)
