% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\name{bb_legend}
\alias{bb_legend}
\title{bb_legend}
\usage{
bb_legend(
  position = "topright",
  title = NULL,
  aesthetic = "col",
  pch = 19,
  bty = "n",
  outside = FALSE,
  ...
)
}
\arguments{
\item{position}{legend position passed to \code{graphics::legend}.}

\item{title}{legend title.}

\item{aesthetic}{currently supports \code{"col"}, \code{"fill"}, \code{"pch"}, \code{"lty"}, \code{"cex"}.}

\item{pch}{point shape used in legend.}

\item{bty}{box type passed to \code{graphics::legend}.}

\item{outside}{logical, whether to place the legend outside the plotting area.}

\item{...}{additional parameters passed to \code{graphics::legend}.}
}
\value{
A modified bbplot object
}
\description{
Add legend for aesthetic mapping.
}
\author{
Guangchuang Yu
}
